@@ -60,7 +60,7 @@ static const QString s_string_entry6{};
 static const char s_utf8bit_entry[] = "Hello äöü";
 static const QString s_translated_string_entry1{QStringLiteral("bonjour")};
 static const QByteArray s_bytearray_entry{"\x00\xff\x7f\x3c abc\x00\x00", 10};
-static const char s_escapekey[] = " []\0017[]==]";
+static const char s_escapekey[] = " []\0017[]==\\]";
 static const char s_escape_entry[] = "[]\170[]]=3=]\\] ";
 static const double s_double_entry{123456.78912345};
 static const float s_float_entry{123.567f};
@@ -69,6 +69,12 @@ static const QSize s_size_entry{10, 20};
 static const QRect s_rect_entry{10, 23, 5321, 13};
 static const QDateTime s_date_time_entry{QDate{2002, 06, 23}, QTime{12, 55, 40}};
 static const QDateTime s_date_time_with_ms_entry{QDate{2002, 06, 23}, QTime{12, 55, 40, 532}};
+static const QDateTime s_date_time_utc = QDateTime::currentDateTimeUtc();
+static const QDateTime s_date_time_tz = [] {
+    auto dt = QDateTime::currentDateTime();
+    dt.setTimeZone(QTimeZone("EST"));
+    return dt;
+}();
 static const QStringList s_stringlist_entry{QStringLiteral("Hello,"), QStringLiteral("World")};
 static const QStringList s_stringlist_empty_entry{};
 static const QStringList s_stringlist_just_empty_element{QString{}};
@@ -162,6 +168,8 @@ void KConfigTest::initTestCase()
     cg.writeEntry("dateTimeEntry", s_date_time_entry);
     cg.writeEntry("dateEntry", s_date_time_entry.date());
     cg.writeEntry("dateTimeWithMSEntry", s_date_time_with_ms_entry);
+    cg.writeEntry("dateTimeUtc", s_date_time_utc);
+    cg.writeEntry("dateTimeTz", s_date_time_tz);
 
     KConfigGroup ct = cg;
     cg = KConfigGroup(&ct, QStringLiteral("Nested Group 1"));
@@ -262,25 +270,6 @@ void KConfigTest::cleanupTestCase()
     }
 }
 
-static QList<QByteArray> readLinesFrom(const QString &path)
-{
-    QFile file(path);
-    const bool opened = file.open(QIODevice::ReadOnly | QIODevice::Text);
-    QList<QByteArray> lines;
-    if (!opened) {
-        qWarning() << "Failed to open" << path;
-        return lines;
-    }
-    QByteArray line;
-    do {
-        line = file.readLine();
-        if (!line.isEmpty()) {
-            lines.append(line);
-        }
-    } while (!line.isEmpty());
-    return lines;
-}
-
 static const QString s_defaultArg = s_test_subdir + QLatin1String("kconfigtest");
 static QList<QByteArray> readLines(const QString &fileName = s_defaultArg)
 {
@@ -516,7 +505,7 @@ void KConfigTest::testPath()
     qputenv("WITHSLASH", "/a/");
     {
         QFile file(m_testConfigDir + QLatin1String("/pathtest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[Test Group]\n"
             << "homePath=$HOME/foo\n"
@@ -577,7 +566,7 @@ void KConfigTest::testPathQtHome()
 {
     {
         QFile file(m_testConfigDir + QLatin1String("/pathtest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[Test Group]\n"
             << "dataDir[$e]=$QT_DATA_HOME/kconfigtest\n"
@@ -620,6 +609,8 @@ void KConfigTest::testComplex()
     QCOMPARE(sc3.readEntry("dateTimeEntry", QDateTime()).toString(Qt::ISODateWithMs), s_date_time_entry.toString(Qt::ISODateWithMs));
     QCOMPARE(sc3.readEntry("dateEntry", QDate()).toString(Qt::ISODate), s_date_time_entry.date().toString(Qt::ISODate));
     QCOMPARE(sc3.readEntry("dateTimeWithMSEntry", QDateTime()).toString(Qt::ISODateWithMs), s_date_time_with_ms_entry.toString(Qt::ISODateWithMs));
+    QCOMPARE(sc3.readEntry("dateTimeTz", QDateTime()).timeZone(), s_date_time_tz.timeZone());
+    QCOMPARE(sc3.readEntry("dateTimeUtc", QDateTime()).timeRepresentation().id(), s_date_time_utc.timeRepresentation().id());
     QCOMPARE(sc3.readEntry("dateTimeEntry", QDate()), s_date_time_entry.date());
 }
 
@@ -1017,7 +1008,7 @@ void KConfigTest::testMerge()
     {
         // simulate writing by another process
         QFile file(m_testConfigDir + QLatin1String("/mergetest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[Merged Group]\n"
             << "entry1=Testing\n"
@@ -1043,7 +1034,7 @@ void KConfigTest::testMerge()
               << "entry[es]=Spanish\n"
               << "entry[fr]=French\n";
         QFile file(m_testConfigDir + QLatin1String("/mergetest"));
-        file.open(QIODevice::ReadOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::ReadOnly | QIODevice::Text));
         for (const QByteArray &line : std::as_const(lines)) {
             QCOMPARE(line, file.readLine());
         }
@@ -1054,7 +1045,7 @@ void KConfigTest::testImmutable()
 {
     {
         QFile file(m_testConfigDir + QLatin1String("/immutabletest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[$i]\n"
             << "entry1=Testing\n"
@@ -1078,7 +1069,7 @@ void KConfigTest::testOptionOrder()
 {
     {
         QFile file(m_testConfigDir + QLatin1String("/doubleattrtest"));
-        file.open(QIODevice::WriteOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::WriteOnly | QIODevice::Text));
         QTextStream out(&file);
         out << "[group3]\n"
             << "entry2=unlocalized\n"
@@ -1104,7 +1095,7 @@ void KConfigTest::testOptionOrder()
               << "entry2[de_DE][$i]=t2\n";
 
         QFile file(m_testConfigDir + QLatin1String("/doubleattrtest"));
-        file.open(QIODevice::ReadOnly | QIODevice::Text);
+        QVERIFY(file.open(QIODevice::ReadOnly | QIODevice::Text));
         for (const QByteArray &line : std::as_const(lines)) {
             QCOMPARE(line, file.readLine());
         }
@@ -1920,7 +1911,50 @@ void KConfigTest::testMoveValuesTo()
 
     targetGroup.sync();
     QFile targetReadFile(targetFile.fileName());
-    targetReadFile.open(QFile::ReadOnly);
+    QVERIFY(targetReadFile.open(QFile::ReadOnly));
+    QVERIFY(targetReadFile.readAll().contains(QByteArray("my_path[$e]=~/somepath")));
+}
+
+void KConfigTest::testMoveValuesToWithNestedGroup()
+{
+    QTemporaryFile file;
+    QVERIFY(file.open());
+    // Prepare kdeglobals
+    {
+        KConfig glob(QStringLiteral("kdeglobals"));
+        KConfigGroup general(&glob, QStringLiteral("TestGroup"));
+        general.writeEntry("GlobalKey", "PlsDeleteMe");
+        QVERIFY(glob.sync());
+    }
+
+    KConfigGroup grp = KSharedConfig::openConfig(file.fileName())->group(QStringLiteral("TestGroup")).group(QStringLiteral("Nested"));
+
+    grp.writeEntry("test1", "first_value");
+    grp.writeEntry("test_empty", "");
+    grp.writeEntry("other", "other_value");
+    grp.writePathEntry("my_path", QStringLiteral("~/somepath"));
+    // because this key is from the global file it should be explicitly deleted
+    grp.deleteEntry("GlobalKey");
+
+    QTemporaryFile targetFile;
+    QVERIFY(targetFile.open());
+    targetFile.close();
+    KConfigGroup targetGroup =
+        KSharedConfig::openConfig(targetFile.fileName(), KConfig::SimpleConfig)->group(QStringLiteral("MoveToGroup")).group(QStringLiteral("AnotherNested"));
+
+    grp.moveValuesTo({"test1", "test_empty", "does_not_exist", "my_path", "GlobalKey"}, targetGroup);
+    QVERIFY(grp.config()->isDirty());
+    QVERIFY(targetGroup.config()->isDirty());
+
+    QCOMPARE(grp.keyList(), QStringList{QStringLiteral("other")});
+    QStringList expectedKeyList{QStringLiteral("my_path"), QStringLiteral("test1"), QStringLiteral("test_empty")};
+
+    QCOMPARE(targetGroup.keyList(), expectedKeyList);
+    QCOMPARE(targetGroup.readEntry("test1"), QStringLiteral("first_value"));
+
+    targetGroup.sync();
+    QFile targetReadFile(targetFile.fileName());
+    QVERIFY(targetReadFile.open(QFile::ReadOnly));
     QVERIFY(targetReadFile.readAll().contains(QByteArray("my_path[$e]=~/somepath")));
 }
 
@@ -1959,7 +1993,7 @@ void KConfigTest::testMoveAllValuesTo()
 
     targetGroup.sync();
     QFile targetReadFile(targetFile.fileName());
-    targetReadFile.open(QFile::ReadOnly);
+    QVERIFY(targetReadFile.open(QFile::ReadOnly));
     QVERIFY(targetReadFile.readAll().contains(QByteArray("my_path[$e]=~/somepath")));
 }
 
