@@ -50,7 +50,6 @@ QString KConfigIniBackend::warningProlog(const QFile &file, int line)
 }
 
 KConfigIniBackend::KConfigIniBackend()
-    : lockFile(nullptr)
 {
 }
 
@@ -254,10 +253,6 @@ KConfigIniBackend::ParseInfo KConfigIniBackend::parseConfig(const QByteArray &cu
                 }
             }
 
-            if (!(entryOptions & KEntryMap::EntryRawKey)) {
-                printableToString(aKey, file, lineNo);
-            }
-
             if (options & ParseGlobal) {
                 entryOptions |= KEntryMap::EntryGlobal;
             }
@@ -293,7 +288,7 @@ KConfigIniBackend::ParseInfo KConfigIniBackend::parseConfig(const QByteArray &cu
     return fileOptionImmutable ? ParseImmutable : ParseOk;
 }
 
-void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file, const KEntryMap &map, bool defaultGroup, bool &firstEntry)
+void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file, const KEntryMap &map, bool defaultGroup, bool primaryGroup, bool &firstEntry)
 {
     QString currentGroup;
     bool groupIsImmutable = false;
@@ -302,6 +297,10 @@ void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file,
         if ((key.mGroup != QStringLiteral("<default>")) == defaultGroup) {
             continue; // skip
         }
+        // Either process the primary group or all others
+        if ((mPrimaryGroup.isNull() || key.mGroup != mPrimaryGroup) == primaryGroup) {
+            continue; // skip
+        }
 
         // the only thing we care about groups is, is it immutable?
         if (key.mKey.isNull()) {
@@ -388,10 +387,15 @@ void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file,
     bool firstEntry = true;
 
     // write default group
-    writeEntries(locale, file, map, true, firstEntry);
+    writeEntries(locale, file, map, true, false, firstEntry);
+
+    if (!mPrimaryGroup.isNull()) {
+        // write the primary group - it needs to be written before all other groups
+        writeEntries(locale, file, map, false, true, firstEntry);
+    }
 
     // write all other groups
-    writeEntries(locale, file, map, false, firstEntry);
+    writeEntries(locale, file, map, false, false, firstEntry);
 }
 
 bool KConfigIniBackend::writeConfig(const QByteArray &locale, KEntryMap &entryMap, WriteOptions options)
@@ -511,29 +515,15 @@ bool KConfigIniBackend::writeConfig(const QByteArray &locale, KEntryMap &entryMa
             return false;
         }
     } else {
-        // Open existing file. *DON'T* create it if it suddenly does not exist!
-#if defined(Q_OS_UNIX) && !defined(Q_OS_ANDROID)
-        int fd = QT_OPEN(QFile::encodeName(filePath()).constData(), O_WRONLY | O_TRUNC);
-        if (fd < 0) {
-            return false;
-        }
-        QFile f;
-        if (!f.open(fd, QIODevice::WriteOnly)) {
-            QT_CLOSE(fd);
-            return false;
-        }
-        writeEntries(locale, f, writeMap);
-        f.close();
-        QT_CLOSE(fd);
-#else
         QFile f(filePath());
-        // XXX This is broken - it DOES create the file if it is suddenly gone.
-        if (!f.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+
+        // Open existing file. *DON'T* create it if it suddenly does not exist!
+        if (!f.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::ExistingOnly)) {
             return false;
         }
+
         f.setTextModeEnabled(true);
         writeEntries(locale, f, writeMap);
-#endif
     }
     return true;
 }
@@ -626,15 +616,15 @@ bool KConfigIniBackend::lock()
             // we can't create file at an arbitrary location, so use internal storage to create one
 
             // NOTE: filename can be the same, but because this lock is short lived we may never have a collision
-            lockFile = new QLockFile(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/")
-                                     + QFileInfo(filePath()).fileName() + QLatin1String(".lock"));
+            lockFile = std::make_unique<QLockFile>(QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1String("/")
+                                                   + QFileInfo(filePath()).fileName() + QLatin1String(".lock"));
         } else {
-            lockFile = new QLockFile(filePath() + QLatin1String(".lock"));
+            lockFile = std::make_unique<QLockFile>(filePath() + QLatin1String(".lock"));
         }
     }
 #else
     if (!lockFile) {
-        lockFile = new QLockFile(filePath() + QLatin1String(".lock"));
+        lockFile = std::make_unique<QLockFile>(filePath() + QLatin1String(".lock"));
     }
 #endif
 
@@ -648,7 +638,6 @@ bool KConfigIniBackend::lock()
 void KConfigIniBackend::unlock()
 {
     lockFile->unlock();
-    delete lockFile;
     lockFile = nullptr;
     m_mutex.unlock();
 }
@@ -957,4 +946,9 @@ void KConfigIniBackend::setLocalFilePath(const QString &file)
     mLocalFilePath = file;
 }
 
+void KConfigIniBackend::setPrimaryGroup(const QString &group)
+{
+    mPrimaryGroup = group;
+}
+
 #include "moc_kconfigini_p.cpp"
